/*
Copyright (c) 2010 Goeran Bruns <goeran@brunsware.de>

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated
documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to
whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
DEALINGS IN THE SOFTWARE.
*/

#include "outgauge_example.h"
#include <iostream>
#include <math.h>
#include <mmsystem.h>

using namespace std;


SOCKET ogSocket; // communication socket

#define OUTGAUGE_ADDRESS "127.0.0.1" // ip to listen
#define OUTGAUGE_PORT 27015 // port to listen
#define SOUND_FLASHER "Flashers.wav"
#define SOUND_DING "Ding.wav"

int main(int argc, char* argv[])
{
	SetConsoleCtrlHandler(CtrlC_HandlerRoutine, TRUE); // handle ctrl + c keypress
	WSADATA wsaData; // winsock data struct
	int iBytesRecv = 0;
	BYTE recvBuf[200]; // receive buffer
 
	// init winsock
	int iRes = WSAStartup(MAKEWORD(2,2), &wsaData);
	if(iRes != NO_ERROR)
	{
		cout << "Error at WSAStartup()" << endl;
		return 0;
	}

	 // init socket
	ogSocket = socket(AF_INET,SOCK_DGRAM,IPPROTO_UDP);
	if(ogSocket == INVALID_SOCKET)
	{
		cout << "Error at socket(): " << WSAGetLastError() << endl;;
	    WSACleanup();
		return 0;
	}

	// bind socket to address:port
	sockaddr_in service;
    service.sin_family = AF_INET;
    service.sin_addr.s_addr = inet_addr(OUTGAUGE_ADDRESS);
    service.sin_port = htons(OUTGAUGE_PORT);
	if (bind( ogSocket, (SOCKADDR*) &service, sizeof(service)) == SOCKET_ERROR)
	{
		cout << "bind() failed." << endl;
		closesocket(ogSocket);
		WSACleanup();
		return 0;
	}

	// main loop
	cout << "Outgauge example" << endl;
	cout << "----------------" << endl;
	while( iBytesRecv != SOCKET_ERROR )
	{
		// read from socket
	    iBytesRecv = recv( ogSocket, (PCHAR)recvBuf, 200, 0 );
		if ( iBytesRecv <= 0 )
		{
			cout << "Connection Closed." << endl;
			break;
		}
		// there is no check if this is a outgauge pack ... !not for production use!
		OutGaugePack* ogPacket = (OutGaugePack*)&recvBuf;
		// write some data
		WriteDataToConsole(ogPacket);
	}
	closesocket(ogSocket);
	WSACleanup();
}

BOOL WINAPI CtrlC_HandlerRoutine ( DWORD dwType )
{ 
	PlaySound(NULL,NULL,SND_ASYNC);
	closesocket(ogSocket);
	WSACleanup();
    return TRUE;  
}

void WriteDataToConsole(const OutGaugePack* ogPacket)
{
	char buf[80];
	memset(buf, 0, 80);
	COORD cursorPos = {0,2};
	char gear;
	float speed = ogPacket->Speed * (float)3.6;
	switch(ogPacket->Gear)
	{
		case 0:
		gear = 'R';
		break;
		case 1:
		gear = 'N';
		break;
		default:
			gear = (char)ogPacket->Gear+47;
	}	
	HANDLE hOutput = GetStdHandle(STD_OUTPUT_HANDLE);
	SetConsoleCursorPosition(hOutput, cursorPos);
	sprintf(buf, "Car: %s | RPM: %5.0f | Speed: %3.0f kph | Gear: %c", ogPacket->Car, ogPacket->RPM, speed, gear);
	WriteConsole(hOutput,buf,sizeof(buf), NULL, NULL);
	memset(buf, 0, 80);
	cursorPos.Y = 3;
	SetConsoleCursorPosition(hOutput, cursorPos);
	buf[0] = 'T';
	buf[1] = ':';
	for(int i=2; i <= (int)(ceil(78.0 * ogPacket->Throttle) + 1);i++)
	{
		buf[i] = (char)219;
	}
	WriteConsole(hOutput,buf,sizeof(buf), NULL, NULL);
	memset(buf, 0, 80);
	cursorPos.Y = 4;
	SetConsoleCursorPosition(hOutput, cursorPos);
	buf[0] = 'B';
	buf[1] = ':';
	for(int i=2; i <= (int)(ceil(78.0 * ogPacket->Brake) + 1);i++)
	{
		buf[i] = (char)219;
	}
	WriteConsole(hOutput,buf,sizeof(buf), NULL, NULL);
	if(ogPacket->ShowLights & 1 << DL_SIGNAL_L || ogPacket->ShowLights & 1 << DL_SIGNAL_R)
	{
		PlaySound(SOUND_FLASHER, NULL, SND_FILENAME | SND_ASYNC | SND_NOSTOP);
	}
	if(ogPacket->ShowLights & 1 << DL_SHIFT)
	{
		PlaySound(SOUND_DING, NULL, SND_FILENAME | SND_ASYNC | SND_NOSTOP);
	}
}